package Template::Plugin::MIB::MembersList;
use base qw( Template::Plugin );

use strict;
use warnings;
use CGI qw(:html -nosticky);
use Net::LDAP;
use Net::LDAP::Util;
use Encode;
use Unicode::Collate;

my $ldap = Net::LDAP->new([qw/ldap.math.uni-bonn.de ldap-back.math.uni-bonn.de/]) or die "$@";
my $mesg = $ldap->bind;
$mesg->code && die $mesg->error;
my $ldap_base = 'ou=people,dc=math,dc=uni-bonn,dc=de';

my $Collator = Unicode::Collate->new;

my @filter = (
	{
		header => 'Professoren',
		filter => '(employeeType=Professor)'
	},
	{
		header => 'Bonn Junior Fellows und IRU',
		filter => '(employeeType=hcm)'
	},
	{
		header => 'Professorenvertreter',
		filter => '(employeeType=ProfVertreter)'
	},
	{
		header => 'Research Chairs und weitere Gastprofessuren',
		filter => '(employeeType=GastProf)'
	},
	{
		header => 'Hausdorff-Edition',
		filter => '(employeeType=Hausdorff)'
	},
	{
		header => 'Emeritierte und pensionierte Professoren',
		filter => '(employeeType=Emeritus)'
	},
	{
	header => 'Akademische R&auml;te',
		filter => '(employeeType=AkadRat)'
	},
	{
	header => 'wissenschaftliche Mitarbeiter',
		filter => '(employeeType=Mitarbeiter)'
	},
	{
	header => 'nicht-wissenschaftliche Mitarbeiter',
		filter => '(employeeType=NWMitarbeiter)'
	},
);

my $filter_alpha = "(|(employeeType=Professor)(employeeType=hcm)(employeeType=ProfVertreter)(employeeType=GastProf)(employeeType=Hausdorff)(employeeType=Emeritus)(employeeType=AkadRat)(employeeType=Mitarbeiter)(employeeType=NWMitarbeiter))";

my @header = ( 'Name', 'Email', 'Zimmer', 'Tel (73-)', 'Portrait' );
my @header_print = ( 'Name', 'Email', 'Zimmer', 'Tel (73-)' );
my @header_homeless = ( 'Name', 'Email', 'Portrait' );
my @header_homeless_print = ( 'Name', 'Email' );

sub add_row {
	my (@data) = @_;
	my ($row);
	for my $col (@data) {
		$row .= td($col);
	}
	return Tr($row);
}

sub add_rows {
	my (@data) = @_;
	my $content;
	foreach my $row (@data) {
		$content .= add_row(@{$row});
	}
	return $content;
}

sub get_rows_for {
	my ($filter, %options) = @_;
	my $homeless = $options{homeless};
	my $print = $options{print};

	my @attrs = qw(mail givenname sn title labeledURI jpegPhoto uid);
	push @attrs, qw(telephoneNumber room) unless $homeless;

	my $mesg = $ldap->search(
		base   => $ldap_base,
		filter => "$filter",
		attrs => \@attrs,
	);
	$mesg->code && die $mesg->error;

	my @entries;
	foreach my $ldap_entry ($mesg->entries()) {
		my %entry;
		# Es ist gefährlich, as mit ( uid => $ldap_entry->get_value('uid'), etc zu initialiieren.
		$entry{uid}       = $ldap_entry->get_value('uid');
		$entry{title}     = Encode::decode('utf8', $ldap_entry->get_value('title'));
		$entry{givenname} = Encode::decode('utf8', $ldap_entry->get_value('givenname'));
		$entry{sn}        = Encode::decode('utf8', $ldap_entry->get_value('sn'));
		$entry{mail}      = $ldap_entry->get_value('mail');
		$entry{uri}       = $ldap_entry->get_value('labeledURI');
		$entry{phone}     = $ldap_entry->get_value('telephoneNumber') unless $homeless;
		$entry{room}      = $ldap_entry->get_value('roomNumber') unless $homeless;
		$entry{has_photo} = $ldap_entry->exists('jpegPhoto');
		push @entries, \%entry;
	}

	my @rows;
	foreach my $entry (sort {
		$Collator->cmp($a->{sn}, $b->{sn}) ||
		$Collator->cmp($a->{givenname}, $b->{givenname}) ||
		$a->{uid} <=> $b->{uid};
	} @entries) {
		my ($name, $mail);

		if ($entry->{title}) {
			$name = "$entry->{sn}, $entry->{title} $entry->{givenname}";
		} else {
			$name = "$entry->{sn}, $entry->{givenname}";
		}

		if ($entry->{uri} && !$print) {
			$name = a( { href => $entry->{uri} }, $name );
		}

		if ($entry->{mail}) {
			$mail = (split(/@/, $entry->{mail}))[0];
			$mail = a({ href => 'mailto:' . $mail}, $mail) unless $print;
		}

		my @cols = ($name, $mail);
		push @cols, ($entry->{room}, $entry->{phone}) unless $homeless;

		if ($entry->{has_photo} && !$print) {
			push @cols,
			  a( { href => "/members?mode=portrait&uid=$entry->{uid}" },
				"Portrait" );
		}
		push @rows, [ @cols ];
	}
	return @rows;
}


sub alpha {
	my (%options) = @_;
	my $print = $options{print};
	my @rows = get_rows_for($filter_alpha, %options);

	my $html;
	$html .= Tr(th({ align => 'left' }, [ $print ? @header_print : @header ]));
	$html .= add_rows(@rows);
	return h2('Mitarbeiter') . table({ width => '100%', class => 'text full' },$html);
}

sub csv {
	my (%options) = @_;
	my @attrs = qw(sn givenname title roomNumber telephoneNumber mail employeeType);
	my $result = "";
	
	my $mesg = $ldap->search(
		base   => $ldap_base,
		filter => $filter_alpha,
		attrs => \@attrs,
	);
	$mesg->code && die $mesg->error;

	foreach my $entry ($mesg->sorted( 'sn', 'givenname')) {
		my $employeeType = $entry->get_value('employeeType');
		my $wiss = 0; my $admin = 0;
		$wiss = 1 if $employeeType =~ m/^(professor|profvertreter|gastprof|hausdorff|hcm|akadrat|emeritus|mitarbeiter)$/i;
		$admin = 1 if $employeeType =~ m/^(akadrat|nwmitarbeiter)$/i;
		$result .= join(', ', map('"' . Encode::decode('utf8',$entry->get_value($_) // '') . '"', @attrs));
		$result .= ", $wiss, $admin\r\n";
	}
	return $result;
}

sub struc {
	my (%options) = @_;
	my $print = $options{print};
	my $html;
	$html .= Tr(th({ align => 'left' }, [ $print ? @header_print : @header ]));
	foreach my $spec (@filter) {
		my @rows = get_rows_for($spec->{filter}, %options);

		$html .= tbody(
			Tr(td({ colspan => scalar @header}, h2({ class => 'tableheader'}, $spec->{header})))
			. add_rows(@rows)
		) if @rows;
	}
	return h2('Mitarbeiter') . table({ width => '100%', class => 'text full' }, $html);
}

sub former {
	my (%options) = @_;
	my $print = $options{print};
	$options{homeless} = 1;
	my @rows = get_rows_for('(employeeType=Ehemaliger)', %options);

	my $html;
	$html .= Tr(th({ align => 'left' }, [ $print ? @header_homeless_print : @header_homeless ]));
	$html .= add_rows(@rows);
	return h2('Ehemalige') . table({ width => '100%', class => 'text full' }, $html);
}

sub future {
	my (%options) = @_;
	my $print = $options{print};
	$options{homeless} = 1;
	my @rows = get_rows_for('(employeeType=Kommender)', %options);

	my $html;
	$html .= Tr(th({ align => 'left' }, [ $print ? @header_homeless_print : @header_homeless ]));
	$html .= add_rows(@rows);
	return h2('Zuk&uuml;nftige') . table({ width => '100%', class => 'text full' }, $html);
}

sub image {
	my $uid = shift;
	return undef unless $uid;
	my $mesg = $ldap->search(
		base   => $ldap_base,
		filter => 'uid=' . Net::LDAP::Util::escape_filter_value($uid),
		attrs => [ qw/jpegPhoto/ ],
	);
	$mesg->code && die $mesg->error;
	return undef if $mesg->count() != 1;
	my $entry = $mesg->entry(0) or return undef;
	return $entry->get_value('jpegPhoto');
}

sub portrait {
	my $uid = shift;
	return undef unless $uid;
	my $filter = '(uid=' . Net::LDAP::Util::escape_filter_value( $uid ) . ')';
	my ($row) = get_rows_for($filter); # Keine options
	return undef unless $row;
	my ( $name, $mail, $room, $phone ) = @{$row};
	my $html;
	return    h2($name)
		. table({ border => 1, class => 'text', width => '100%' },
			Tr(
				td({ align => 'center', width => '50%'}, img({ src => "/image/?uid=$uid", alt => "Bild von $name"})),
				td(
					table(
						Tr(th({ align => 'left'}, 'Name')),
						Tr(th($name)),

						Tr(th({ align => 'left'}, 'email-Adresse')),
						Tr(th($mail)),

						Tr(th({ align => 'left'}, 'Zimmer')),
						Tr(th($room)),

						Tr(th({ align => 'left'}, 'Telefon (73-)')),
						Tr(th($phone)),
					)
				)
			  )
		);
}


sub process {
	my ($self,$cgi,$encoding) = @_;
	my %options;
	my $mode = $cgi->param('mode') || '';
	$options{print} = $cgi->param('print') || 0;
	$options{encoding} = $encoding if $encoding;
	foreach ($mode) {
		/^alpha$/ && return alpha(%options);
		/^struc$/ && return struc(%options);
		/^former$/ && return former(%options);
		/^future$/ && return future(%options);
		/^csv$/ && return csv(%options);
		/^image$/ && return image($cgi->param('uid'));
		/^portrait$/ && return portrait($cgi->param('uid'));
		return alpha(%options);
	}
}

1;
